@@ -1,6 +1,6 @@
 import logging
 from functools import partial
-from typing import Any, Dict
+from typing import TYPE_CHECKING, Any, Dict
 from warnings import warn
 
 import requests
@@ -12,6 +12,9 @@ from sdmx.rest import URL, Resource
 from sdmx.session import ResponseIO, Session
 from sdmx.source import NoSource, list_sources, sources
 
+if TYPE_CHECKING:
+    import sdmx.source
+
 log = logging.getLogger(__name__)
 
 
@@ -52,10 +55,10 @@ class Client:
     cache: Dict[str, Message] = {}
 
     #: :class:`.source.Source` for requests sent from the instance.
-    source = None
+    source: "sdmx.source.Source"
 
     #: :class:`.Session` for queries sent from the instance.
-    session = None
+    session: requests.Session
 
     # Stored keyword arguments "allow_redirects" and "timeout" for pre-requests.
     _send_kwargs: Dict[str, Any] = {}
@@ -167,8 +170,9 @@ class Client:
 
         return cc.to_query_string(dsd), dsd
 
-    def _request_from_args(self, kwargs):
+    def _request_from_args(self, kwargs):  # noqa: C901
         """Validate arguments and prepare pieces for a request."""
+        # TODO Simplify this method to reduce its McCabe complexity from 16 to <= 13
         parameters = kwargs.pop("params", {})
         headers = kwargs.pop("headers", {})
 
@@ -343,8 +347,8 @@ class Client:
            create properly formatted strings.
 
         For formats 1 and 2, but not 3, the `key` argument is validated against the
-        relevant :class:`.DataStructureDefinition`, either given with the `dsd` keyword
-        argument, or retrieved from the web service before the main query.
+        relevant :class:`DSD <.BaseDataStructureDefinition>`, either given with the
+        `dsd` keyword argument, or retrieved from the web service before the main query.
 
         For the optional `param` keyword argument, some useful parameters are:
 
@@ -372,7 +376,7 @@ class Client:
 
         Other Parameters
         ----------------
-        dsd : :class:`~.DataStructureDefinition`
+        dsd : :class:`DataStructureDefinition <.BaseDataStructureDefinition>`
             Existing object used to validate the `key` argument. If not provided, an
             additional query executed to retrieve a DSD in order to validate the `key`.
         force : bool
@@ -501,7 +505,8 @@ class Client:
 
         For the Dataflow `flow_id`, return all series keys matching `key`. Uses a
         feature supported by some data providers that returns :class:`SeriesKeys
-        <.SeriesKey>` without the corresponding :class:`Observations <.Observation>`.
+        <.SeriesKey>` without the corresponding
+        :class:`Observations <.BaseObservation>`.
 
         To count the number of series::
 
