@@ -7,36 +7,49 @@
 data sources.
 """
 import logging
+from dataclasses import dataclass, field, fields
 from datetime import datetime
-from typing import Any, List, Optional, Text, Union
+from operator import attrgetter
+from typing import TYPE_CHECKING, List, Optional, Text, Type, Union, get_args
 
-from sdmx.model import v21 as model
-from sdmx.util import (
-    BaseModel,
-    DictLike,
-    compare,
-    dictlike_field,
-    direct_fields,
-    get_args,
-    summarize_dictlike,
-    validate_dictlike,
+from sdmx import model
+from sdmx.dictlike import DictLike, DictLikeDescriptor, summarize_dictlike
+from sdmx.format import Version
+from sdmx.model import common, v21, v30
+from sdmx.model.internationalstring import (
+    InternationalString,
+    InternationalStringDescriptor,
 )
+from sdmx.util import compare, direct_fields
+
+if TYPE_CHECKING:
+    import requests
 
 log = logging.getLogger(__name__)
 
 
-def _summarize(obj, fields):
+def _summarize(obj, include: Optional[List[str]] = None):
     """Helper method for __repr__ on Header and Message (sub)classes."""
-    for name in fields:
+    import requests
+
+    include = include or list(map(attrgetter("name"), fields(obj)))
+    for name in include:
         attr = getattr(obj, name)
         if attr is None:
             continue
         elif isinstance(attr, datetime):
-            attr = attr.isoformat()
-        yield f"{name}: {repr(attr)}"
+            s_attr = repr(attr.isoformat())
+        elif isinstance(attr, requests.Response):
+            # Don't use repr(), which displays the entire response body
+            s_attr = str(attr)
+        else:
+            s_attr = repr(attr)
+
+        yield f"{name}: {s_attr}"
 
 
-class Header(BaseModel):
+@dataclass
+class Header:
     """Header of an SDMX-ML message.
 
     SDMX-JSON messages do not have headers.
@@ -60,14 +73,14 @@ class Header(BaseModel):
     #: The :class:`.Agency` associated with the data :class:`~.source.Source`.
     sender: Optional[model.Agency] = None
     #:
-    source: model.InternationalString = model.InternationalString()
+    source: InternationalStringDescriptor = InternationalStringDescriptor()
     #:
     test: bool = False
 
     def __repr__(self):
         """String representation."""
         lines = ["<Header>"]
-        lines.extend(_summarize(self, self.__fields__.keys()))
+        lines.extend(_summarize(self))
         return "\n  ".join(lines)
 
     def compare(self, other, strict=True):
@@ -80,10 +93,14 @@ class Header(BaseModel):
         strict : bool, optional
             Passed to :func:`.compare`.
         """
-        return all(compare(f, self, other, strict) for f in self.__fields__.keys())
+        return all(
+            compare(f, self, other, strict)
+            for f in map(attrgetter("name"), fields(self))
+        )
 
 
-class Footer(BaseModel):
+@dataclass
+class Footer:
     """Footer of an SDMX-ML message.
 
     SDMX-JSON messages do not have footers.
@@ -92,10 +109,17 @@ class Footer(BaseModel):
     #:
     severity: Optional[str] = None
     #: The body text of the Footer contains zero or more blocks of text.
-    text: List[model.InternationalString] = []
+    text: List[model.InternationalString] = field(default_factory=list)
     #:
     code: Optional[int] = None
 
+    def __post_init__(self):
+        # Convert non-IS members to IS
+        self.text = [
+            t if isinstance(t, InternationalString) else InternationalString(t)
+            for t in self.text
+        ]
+
     def compare(self, other, strict=True):
         """Return :obj:`True` if `self` is the same as `other`.
 
@@ -107,21 +131,24 @@ class Footer(BaseModel):
         strict : bool, optional
             Passed to :func:`.compare`.
         """
-        return all(compare(f, self, other, strict) for f in self.__fields__.keys())
+        return all(
+            compare(f, self, other, strict)
+            for f in map(attrgetter("name"), fields(self))
+        )
 
 
-class Message(BaseModel):
-    class Config:
-        # for .response
-        arbitrary_types_allowed = True
+@dataclass
+class Message:
+    #: SDMX version.
+    version: Version = Version["2.1"]
 
     #: :class:`Header` instance.
-    header: Header = Header()
+    header: Header = field(default_factory=Header)
     #: (optional) :class:`Footer` instance.
     footer: Optional[Footer] = None
     #: :class:`requests.Response` instance for the response to the HTTP request that
     #: returned the Message. This is not part of the SDMX standard.
-    response: Optional[Any] = None
+    response: Optional["requests.Response"] = None
 
     def __str__(self):
         return repr(self)
@@ -156,28 +183,75 @@ class ErrorMessage(Message):
     pass
 
 
-@validate_dictlike
+@dataclass
 class StructureMessage(Message):
     #: Collection of :class:`.Categorisation`.
-    categorisation: DictLike[str, model.Categorisation] = dictlike_field()
+    categorisation: DictLikeDescriptor[str, model.Categorisation] = DictLikeDescriptor()
     #: Collection of :class:`.CategoryScheme`.
-    category_scheme: DictLike[str, model.CategoryScheme] = dictlike_field()
+    category_scheme: DictLikeDescriptor[
+        str, model.CategoryScheme
+    ] = DictLikeDescriptor()
     #: Collection of :class:`.Codelist`.
-    codelist: DictLike[str, model.Codelist] = dictlike_field()
+    codelist: DictLikeDescriptor[str, model.Codelist] = DictLikeDescriptor()
+    #: Collection of :class:`.HierarchicalCodelist`.
+    hierarchical_codelist: DictLikeDescriptor[
+        str, v21.HierarchicalCodelist
+    ] = DictLikeDescriptor()
+    #: Collection of :class:`.v30.Hierarchy`.
+    hierarchy: DictLikeDescriptor[str, v30.Hierarchy] = DictLikeDescriptor()
     #: Collection of :class:`.ConceptScheme`.
-    concept_scheme: DictLike[str, model.ConceptScheme] = dictlike_field()
+    concept_scheme: DictLikeDescriptor[str, model.ConceptScheme] = DictLikeDescriptor()
     #: Collection of :class:`.ContentConstraint`.
-    constraint: DictLike[str, model.ContentConstraint] = dictlike_field()
-    #: Collection of :class:`.DataflowDefinition`.
-    dataflow: DictLike[str, model.DataflowDefinition] = dictlike_field()
-    #: Collection of :class:`.DataflowDefinition`.
-    metadataflow: DictLike[str, model.MetadataflowDefinition] = dictlike_field()
-    #: Collection of :class:`.DataStructureDefinition`.
-    structure: DictLike[str, model.DataStructureDefinition] = dictlike_field()
-    #: Collection of :class:`.AgencyScheme`.
-    organisation_scheme: DictLike[str, model.AgencyScheme] = dictlike_field()
+    constraint: DictLikeDescriptor[str, model.BaseConstraint] = DictLikeDescriptor()
+    #: Collection of :class:`Dataflow(Definition) <.BaseDataflow>`.
+    dataflow: DictLikeDescriptor[str, model.BaseDataflow] = DictLikeDescriptor()
+    #: Collection of :class:`MetadataStructureDefinition
+    #: <.BaseMetadataStructureDefinition>`.
+    metadatastructure: DictLikeDescriptor[
+        str, model.BaseMetadataStructureDefinition
+    ] = DictLikeDescriptor()
+    #: Collection of :class:`Metadataflow(Definition) <.BaseMetadataflow>`.
+    metadataflow: DictLikeDescriptor[str, model.BaseMetadataflow] = DictLikeDescriptor()
+    #: Collection of :class:`DataStructureDefinition <.BaseDataStructureDefinition>`.
+    structure: DictLikeDescriptor[
+        str, model.BaseDataStructureDefinition
+    ] = DictLikeDescriptor()
+    #: Collection of :class:`.StructureSet`.
+    structureset: DictLikeDescriptor[str, v21.StructureSet] = DictLikeDescriptor()
+    #: Collection of :class:`.OrganisationScheme`.
+    organisation_scheme: DictLikeDescriptor[
+        str, model.OrganisationScheme
+    ] = DictLikeDescriptor()
     #: Collection of :class:`.ProvisionAgreement`.
-    provisionagreement: DictLike[str, model.ProvisionAgreement] = dictlike_field()
+    provisionagreement: DictLikeDescriptor[
+        str, model.ProvisionAgreement
+    ] = DictLikeDescriptor()
+
+    #: Collection of :class:`.CustomTypeScheme`.
+    custom_type_scheme: DictLikeDescriptor[
+        str, model.CustomTypeScheme
+    ] = DictLikeDescriptor()
+    #: Collection of :class:`.NamePersonalisationScheme`.
+    name_personalisation_scheme: DictLikeDescriptor[
+        str, model.NamePersonalisationScheme
+    ] = DictLikeDescriptor()
+    #: Collection of :class:`.RulesetScheme`.
+    ruleset_scheme: DictLikeDescriptor[str, model.RulesetScheme] = DictLikeDescriptor()
+    #: Collection of :class:`.VTLMappingScheme`.
+    vtl_mapping_scheme: DictLikeDescriptor[
+        str, model.VTLMappingScheme
+    ] = DictLikeDescriptor()
+    #: Collection of :class:`.TransformationScheme`.
+    transformation_scheme: DictLikeDescriptor[
+        str, model.TransformationScheme
+    ] = DictLikeDescriptor()
+    #: Collection of :class:`.UserDefinedOperatorScheme`.
+    user_defined_operator_scheme: DictLikeDescriptor[
+        str, model.UserDefinedOperatorScheme
+    ] = DictLikeDescriptor()
+
+    #: Collection of :class:`.ValueList` (SDMX 3.0 only).
+    valuelist: DictLikeDescriptor[str, v30.ValueList] = DictLikeDescriptor()
 
     def compare(self, other, strict=True):
         """Return :obj:`True` if `self` is the same as `other`.
@@ -191,19 +265,16 @@ class StructureMessage(Message):
             Passed to :meth:`.DictLike.compare`.
         """
         return super().compare(other, strict) and all(
-            getattr(self, f).compare(getattr(other, f), strict)
-            for f in direct_fields(self.__class__).keys()
+            getattr(self, f.name).compare(getattr(other, f.name), strict)
+            for f in direct_fields(self.__class__)
         )
 
     def add(self, obj: model.IdentifiableArtefact):
         """Add `obj` to the StructureMessage."""
-        for field, field_info in direct_fields(self.__class__).items():
+        for f in direct_fields(self.__class__):
             # NB for some reason mypy complains here, but not in __contains__(), below
-            if isinstance(
-                obj,
-                get_args(field_info.outer_type_)[1],  # type: ignore [attr-defined]
-            ):
-                getattr(self, field)[obj.id] = obj
+            if isinstance(obj, get_args(f.type)[1]):
+                getattr(self, f.name)[obj.id] = obj
                 return
         raise TypeError(type(obj))
 
@@ -241,8 +312,8 @@ class StructureMessage(Message):
             filter(
                 None,
                 map(
-                    lambda f: getattr(self, f).get(id),
-                    direct_fields(self.__class__).keys(),
+                    lambda f: getattr(self, f.name).get(id),
+                    direct_fields(self.__class__),
                 ),
             )
         )
@@ -252,22 +323,27 @@ class StructureMessage(Message):
 
         return candidates[0] if len(candidates) == 1 else None
 
+    def iter_collections(self):
+        """Iterate over collections."""
+        for f in direct_fields(self.__class__):
+            yield f.name, get_args(f.type)[1]
+
     def objects(self, cls):
         """Get a reference to the attribute for objects of type `cls`.
 
         For example, if `cls` is the class :class:`DataStructureDefinition` (not an
         instance), return a reference to :attr:`structure`.
         """
-        for name, info in direct_fields(self.__class__).items():
-            if issubclass(cls, info.sub_fields[0].type_):
-                return getattr(self, name)
+        for f in direct_fields(self.__class__):
+            if issubclass(cls, get_args(f.type)[1]):
+                return getattr(self, f.name)
         raise TypeError(cls)
 
     def __contains__(self, item):
         """Return :obj:`True` if `item` is in the StructureMessage."""
-        for field, field_info in direct_fields(self.__class__).items():
-            if isinstance(item, get_args(field_info.outer_type_)[1]):
-                return item in getattr(self, field).values()
+        for f in direct_fields(self.__class__):
+            if isinstance(item, get_args(f.type)[1]):
+                return item in getattr(self, f.name).values()
         raise TypeError(f"StructureMessage has no collection of {type(item)}")
 
     def __repr__(self):
@@ -282,8 +358,9 @@ class StructureMessage(Message):
         return "\n  ".join(lines)
 
 
+@dataclass
 class DataMessage(Message):
-    """Data Message.
+    """SDMX Data Message.
 
     .. note:: A DataMessage may contain zero or more :class:`.DataSet`, so
        :attr:`data` is a list. To retrieve the first (and possibly only)
@@ -292,9 +369,9 @@ class DataMessage(Message):
     """
 
     #: :class:`list` of :class:`.DataSet`.
-    data: List[model.DataSet] = []
+    data: List[model.BaseDataSet] = field(default_factory=list)
     #: :class:`.DataflowDefinition` that contains the data.
-    dataflow: model.DataflowDefinition = model.DataflowDefinition()
+    dataflow: Optional[model.BaseDataflow] = None
     #: The "dimension at observation level".
     observation_dimension: Optional[
         Union[
@@ -304,12 +381,28 @@ class DataMessage(Message):
         ]
     ] = None
 
+    def __post_init__(self):
+        if self.dataflow is None:
+            # Create a default of the appropriate class
+            self.dataflow = {
+                Version["2.1"]: v21.DataflowDefinition,
+                Version["3.0.0"]: v30.Dataflow,
+            }[self.version]()
+
     # Convenience access
     @property
     def structure(self):
         """DataStructureDefinition used in the :attr:`dataflow`."""
         return self.dataflow.structure
 
+    @property
+    def structure_type(self) -> Type[common.Structure]:
+        """:class:`.Structure` subtype describing the contained (meta)data."""
+        return {
+            Version["2.1"]: v21.DataStructureDefinition,
+            Version["3.0.0"]: v30.DataStructureDefinition,
+        }[self.version]
+
     def __repr__(self):
         """String representation."""
         lines = [super().__repr__()]
@@ -343,3 +436,15 @@ class DataMessage(Message):
             and len(self.data) == len(other.data)
             and all(ds[0].compare(ds[1], strict) for ds in zip(self.data, other.data))
         )
+
+
+@dataclass
+class MetadataMessage(DataMessage):
+    """SDMX Metadata Message."""
+
+    @property
+    def structure_type(self) -> Type[common.Structure]:
+        return {
+            Version["2.1"]: v21.MetadataStructureDefinition,
+            Version["3.0.0"]: v30.MetadataStructureDefinition,
+        }[self.version]
