@@ -1,3 +1,4 @@
+import logging
 from collections.abc import Mapping
 
 import pandas as pd
@@ -26,9 +27,13 @@ class StructuredMessageTest(MessageTest):
         yield sdmx.read_sdmx(path / self.filename, dsd=dsd)
 
     # Tests for every class
-    def test_msg(self, path, dsd):
+    def test_msg(self, caplog, path, dsd):
         # The message can be parsed
-        sdmx.read_sdmx(path / self.filename, dsd=dsd)
+        with caplog.at_level(logging.WARNING):
+            sdmx.read_sdmx(path / self.filename, dsd=dsd)
+
+        # No warnings are emitted per missing dsd= argument (khaeru/sdmx#146)
+        assert [] == caplog.messages
 
     def test_structured_by(self, dsd, msg):
         # The DSD was used to parse the message
@@ -123,7 +128,7 @@ class TestSeriesDataSet(StructuredMessageTest):
         s3 = sdmx.to_pandas(data.series[3], attributes="")
         assert isinstance(s3, pd.Series)
         # With expected values
-        assert s3[0] == 1.2894
+        assert s3.iloc[0] == 1.2894
 
         # Single series can be converted with attributes
         s3_attr = sdmx.to_pandas(data.series[3], attributes="osgd")
@@ -203,19 +208,31 @@ class TestSeriesData_SiblingGroup_TS(StructuredMessageTest):
         assert len(data.series) == 4
         g2 = list(data.group.keys())[2]
         assert g2.CURRENCY == "JPY"
-        print(list(data.group.keys()), g2, g2.attrib, sep="\n")
         assert g2.attrib.TITLE == "ECB reference exchange rate, Japanese yen/Euro"
-        # Check group attributes of a series
+
+        # Check direct attributes of SeriesKey
         s = list(data.series)[0]
+        assert 4 == len(s.attrib)
+
+        # Check group attributes of a series
         g_attrib = s.group_attrib
         assert isinstance(g_attrib, Mapping)
-        assert len(g_attrib) == 1
+        assert 1 == len(g_attrib)
 
 
 class TestSeriesData_RateGroup_TS(StructuredMessageTest):
     filename = "rg-ts-ss.xml"
     dsd_filename = "rg-structure.xml"
 
+    def test_dsd(self, dsd):
+        ad = dsd.attributes
+        for id in ("CONF_STATUS_OBS", "OBS_STATUS"):
+            assert isinstance(ad.get(id).related_to, model.PrimaryMeasureRelationship)
+        for id in ("COLL_METHOD", "TITLE"):
+            assert isinstance(ad.get(id).related_to, model.GroupRelationship)
+        for id in ("DECIMALS", "UNIT_MEASURE", "UNIT_MULT"):
+            assert isinstance(ad.get(id).related_to, model.DimensionRelationship)
+
     def test_groups(self, msg):
         data = msg.data[0]
         assert len(data.group) == 5
@@ -225,8 +242,9 @@ class TestSeriesData_RateGroup_TS(StructuredMessageTest):
         assert (
             g2.attrib.TITLE == "ECB reference exchange rate, U.K. Pound sterling /Euro"
         )
+
         # Check group attributes of a series
         s = list(data.series)[0]
         g_attrib = s.group_attrib
         assert isinstance(g_attrib, Mapping)
-        assert len(g_attrib) == 5
+        assert 5 == len(g_attrib)
