@@ -13,9 +13,9 @@ import pytest
 
 import sdmx
 from sdmx import Client
+from sdmx.dictlike import DictLike
 from sdmx.model.v21 import GenericDataSet
 from sdmx.testing import assert_pd_equal
-from sdmx.util import DictLike
 
 
 @pytest.mark.network
@@ -116,7 +116,7 @@ def test_doc_usage_structure():
     msg1 = ecb.categoryscheme(provider="all")
 
     assert msg1.response.url == (
-        "https://sdw-wsrest.ecb.europa.eu/service/categoryscheme/all/all/latest"
+        "https://data-api.ecb.europa.eu/service/categoryscheme/all/all/latest"
         "?references=parentsandsiblings"
     )
 
