@@ -13,7 +13,7 @@ import requests_mock
 
 import sdmx
 from sdmx import Client
-from sdmx.exceptions import HTTPError, SSLError, XMLParseError
+from sdmx.exceptions import HTTPError, XMLParseError
 
 # Mark the whole file so the tests can be excluded/included
 pytestmark = pytest.mark.source
@@ -33,15 +33,7 @@ class DataSourceTest:
     source_id: str
 
     #: Failures affecting **all** data sources, internal to :mod:`sdmx`.
-    xfail_common = {
-        "actualconstraint": (XMLParseError, NI),  # KeyError
-        "allowedconstraint": (XMLParseError, NI),  # KeyError
-        "contentconstraint": (XMLParseError, NI),  # KeyError
-        "hierarchicalcodelist": (XMLParseError, NI),  # <str:HierarchicalCodelist>
-        "metadatastructure": (XMLParseError, NI),  # <str:MetadataStructure> not parsed
-        "structure": (XMLParseError, NI),  # <str:StructureSet> not parsed
-        "structureset": (XMLParseError, NI),  # <str:StructureSet> not implemented
-    }
+    xfail_common: Dict[str, Any] = {}
 
     #: Mapping of endpoint → Exception subclass. Tests of these endpoints are expected
     #: to fail with the given kind of exception.
@@ -77,28 +69,54 @@ class DataSourceTest:
     #     )
 
     @pytest.mark.network
-    def test_endpoint(self, cache_path, client, endpoint, args):
+    def test_endpoint(self, pytestconfig, cache_path, client, endpoint, args):
         # See sdmx.testing._generate_endpoint_tests() for values of `endpoint`
         cache = cache_path.with_suffix(f".{endpoint}.xml")
 
         try:
-            result = client.get(endpoint, tofile=cache, **args)
+            message = client.get(endpoint, tofile=cache, **args)
         except HTTPError as e:  # For debugging/test development
             print(e)
             raise
 
+        if pytestconfig.getoption("verbose"):
+            print(repr(message))
+            if endpoint == "dataflow":
+                # Display the IDs of data flows; this can be used to identify targets
+                # for tests of the data endpoint for a particular source
+                for dfd in message.dataflow:
+                    print(repr(dfd))
+
         # For debugging
-        # print(cache, cache.read_text(), result, sep='\n\n')
+        # print(cache, cache.read_text(), result, sep="\n\n")
         # assert False
 
-        sdmx.to_pandas(result)
-
-        del result
+        # All parsed contents can also be converted to pandas
+        sdmx.to_pandas(message)
 
 
 class TestABS(DataSourceTest):
     source_id = "ABS"
 
+    endpoint_args = {
+        "data": dict(
+            resource_id="ABS,ANA_AGG,1.0.0",
+            key="....Q",
+            params=dict(startPeriod="2020-Q1", endPeriod="2022-Q4"),
+        )
+    }
+
+
+class TestABS_JSON(DataSourceTest):
+    source_id = "ABS_JSON"
+
+    endpoint_args = {
+        "data": dict(
+            resource_id="ABS,ANA_AGG,1.0.0",
+            key="....Q",
+        )
+    }
+
 
 class TestBBK(DataSourceTest):
     source_id = "BBK"
@@ -130,6 +148,10 @@ class TestBBK(DataSourceTest):
 class TestBIS(DataSourceTest):
     source_id = "BIS"
 
+    endpoint_args = {
+        "actualconstraint": dict(resource_id="CBP_D_24D"),
+    }
+
 
 class TestECB(DataSourceTest):
     source_id = "ECB"
@@ -237,6 +259,41 @@ class TestESTAT(DataSourceTest):
         assert cl4.is_partial and 0 < len(cl4) < len(cl3)
 
 
+class TestESTAT_COMEXT(DataSourceTest):
+    source_id = "ESTAT_COMEXT"
+
+    endpoint_args = {
+        "data": dict(
+            resource_id="DS-059271",
+            params=dict(startPeriod="2023"),
+        ),
+    }
+
+
+class TestCOMP(DataSourceTest):
+    source_id = "COMP"
+
+    endpoint_args = {
+        "data": dict(resource_id="AID_RAIL"),
+    }
+
+
+class TestEMPL(DataSourceTest):
+    source_id = "EMPL"
+
+    endpoint_args = {
+        "data": dict(resource_id="LMP_IND_EXP"),
+    }
+
+
+class TestGROW(DataSourceTest):
+    source_id = "GROW"
+
+    endpoint_args = {
+        "data": dict(resource_id="POST_CUBE1_X"),
+    }
+
+
 class TestILO(DataSourceTest):
     source_id = "ILO"
     xfail = {
@@ -302,6 +359,9 @@ class TestISTAT(DataSourceTest):
         "organisationscheme": HTTPError,  # 400
         "structure": NotImplementedError,  # 501
     }
+    endpoint_args = {
+        "actualconstraint": dict(resource_id="CONS_92_143"),
+    }
 
     @pytest.mark.network
     def test_gh_75(self, client):
@@ -418,11 +478,18 @@ class TestNBB(DataSourceTest):
 
 class TestOECD(DataSourceTest):
     source_id = "OECD"
-
-    xfail = {
-        "data": (SSLError, "SSL: UNSAFE_LEGACY_RENEGOTIATION_DISABLED"),
+    endpoint_args = {
+        "actualconstraint": dict(resource_id="CR_A_DSD_DEBT_TRANS_COLL@DF_MICRO"),
+        "data": dict(
+            resource_id="DSD_MSTI@DF_MSTI",
+            headers={"Accept-Encoding": "compress, gzip"},
+        ),
     }
 
+
+class TestOECD_JSON(DataSourceTest):
+    source_id = "OECD_JSON"
+
     endpoint_args = {
         "data": dict(
             resource_id="ITF_GOODS_TRANSPORT",
@@ -433,6 +500,13 @@ class TestOECD(DataSourceTest):
         )
     }
 
+    @pytest.fixture
+    def client(self, cache_path):
+        # Same as the default implementation, only using oecd_json.Client
+        from sdmx.source.oecd_json import Client
+
+        return Client(self.source_id, cache_name=str(cache_path), backend="sqlite")
+
 
 class TestSGR(DataSourceTest):
     source_id = "SGR"
@@ -445,11 +519,12 @@ class TestSPC(DataSourceTest):
         "structure": NotImplementedError,  # 501
     }
     endpoint_args = {
+        "actualconstraint": dict(resource_id="CR_A_DF_ADBKI"),
         "data": dict(
             resource_id="DF_CPI",
             key="A.CK+FJ..",
             params=dict(startPeriod=2010, endPeriod=2015),
-        )
+        ),
     }
 
 
