@@ -1,21 +1,21 @@
 import pickle
+from dataclasses import dataclass
 from typing import Generator, Type
 
-import pydantic
 import pytest
-from pydantic import StrictStr
 
 import sdmx
-from sdmx.util import BaseModel, DictLike, only, parse_content_type, validate_dictlike
+from sdmx.dictlike import DictLike, DictLikeDescriptor
+from sdmx.util import only, parse_content_type
 
 
 class TestDictLike:
     @pytest.fixture
     def Foo(self) -> Generator[Type, None, None]:
         # Example class
-        @validate_dictlike
-        class Foo(BaseModel):
-            items: DictLike[StrictStr, int] = DictLike()
+        @dataclass
+        class Foo:
+            items: DictLikeDescriptor[str, int] = DictLikeDescriptor()
 
         yield Foo
 
@@ -53,20 +53,10 @@ class TestDictLike:
         assert isinstance(copied, DictLike)
         assert copied.TIME_PERIOD == dl.TIME_PERIOD
 
-    def test_validate_dictlike(self, Foo):
-        """``@validate_dictlike()`` adds a validator to a pydantic model field."""
-        # At least 1 validator is added, with the name _validate_whole
-        assert 1 <= len(Foo.__fields__["items"].post_validators)
-        assert "_validate_whole" == Foo.__fields__["items"].post_validators[0].__name__
-
-        # ValidationError
-        f = Foo()
-        with pytest.raises(pydantic.ValidationError, match="not a valid dict"):
-            f.items = {"bar", "baz"}
-
     def test_validation(self, Foo) -> None:
         f = Foo()
         assert type(f.items) is DictLike
+        assert (str, int) == f.items._types
 
         # Can be set with DictLike
         f.items = DictLike(a=1, b=2)
@@ -77,29 +67,23 @@ class TestDictLike:
         assert type(f.items) is DictLike
 
         # Type checking on creation
-        with pytest.raises(pydantic.ValidationError):
+        with pytest.raises(TypeError):
             f = Foo(items={1: "a"})
 
         # Type checking on assignment
         f = Foo()
-        with pytest.raises(pydantic.ValidationError):
+        with pytest.raises(TypeError):
             f.items = {1: "a"}
 
         # Type checking on setting elements
         f = Foo(items={"a": 1})
-        with pytest.raises(pydantic.ValidationError):
+        with pytest.raises(TypeError):
             f.items[123] = 456
 
-        # commented: this does not work, since validate_dictlike does not operate
-        # until initial values are assigned to the field
-        # f = Foo()
-        # with pytest.raises(pydantic.ValidationError):
-        #     f.items[123] = 456
-
-        # Use validate_dictlike() twice
-        @validate_dictlike
-        class Bar(BaseModel):
-            elems: DictLike[StrictStr, float] = DictLike()
+        # With no initial value set
+        f = Foo()
+        with pytest.raises(TypeError):
+            f.items[123] = 456
 
     def test_compare(self, caplog):
         dl1 = DictLike(a="foo", b="bar")
