@@ -1,8 +1,11 @@
+import io
 import logging
 
 import pytest
+from lxml import etree
 
 import sdmx
+import sdmx.writer.xml
 from sdmx import message
 from sdmx.model import v21 as m
 from sdmx.model.v21 import DataSet, DataStructureDefinition, Dimension, Key, Observation
@@ -47,7 +50,11 @@ def dks(dsd):
 
 def test_contact() -> None:
     c = m.Contact(
-        name="John Smith", org_unit="Human Resources", telephone="+1234567890"
+        name="John Smith",
+        org_unit="Human Resources",
+        telephone="+1234567890",
+        uri=["https://example.org"],
+        email=["john.smith@example.org"],
     )
 
     result = sdmx.to_xml(c, pretty_print=True)
@@ -57,6 +64,8 @@ def test_contact() -> None:
   <com:Name xml:lang="en">John Smith</com:Name>
   <str:Department xml:lang="en">Human Resources</str:Department>
   <str:Telephone>+1234567890</str:Telephone>
+  <str:URI>https://example.org</str:URI>
+  <str:Email>john.smith@example.org</str:Email>
 </str:Contact>
 """
     )
@@ -83,13 +92,14 @@ def test_ContentConstraint(dsd, dks):
     )
 
 
-def test_ds(dsd, obs):
+def test_ds(dsd, obs) -> None:
     # Write DataSet with Observations not in Series
     ds = DataSet(structured_by=dsd)
     ds.obs.append(obs)
 
     result = sdmx.to_xml(ds, pretty_print=True)
-    print(result.decode())
+    # print(result.decode())
+    del result
 
 
 def test_ds_structurespecific(dsd):
@@ -138,6 +148,24 @@ def test_obs(obs):
         XMLWriter.recurse(obs, struct_spec=True)
 
 
+def test_reference() -> None:
+    cl = m.Codelist(id="FOO", version="1.0")
+    c = m.Code(id="BAR")
+    cl.append(c)
+
+    # <Ref …> to Item has maintainableParentVersion, but no version
+    result = sdmx.writer.xml.reference(c, style="Ref")
+    result_str = etree.tostring(result).decode()
+    assert 'maintainableParentVersion="1.0"' in result_str
+    assert 'version="1.0"' not in result_str
+
+    # <Ref …> to ItemScheme has version, but not maintainableParentVersion
+    result = sdmx.writer.xml.reference(cl, style="Ref")
+    result_str = etree.tostring(result).decode()
+    assert 'maintainableParentVersion="1.0"' not in result_str
+    assert 'version="1.0"' in result_str
+
+
 def test_Footer(footer):
     """:class:`.Footer` can be written."""
     sdmx.to_xml(footer)
@@ -177,12 +205,6 @@ def test_ErrorMessage(errormessage):
     sdmx.to_xml(errormessage)
 
 
-_xf_ref = pytest.mark.xfail(
-    raises=NotImplementedError, reason="Cannot write reference to .* without parent"
-)
-_xf_not_equal = pytest.mark.xfail(raises=AssertionError)
-
-
 @pytest.mark.parametrize(
     "data_id, structure_id",
     [
@@ -241,29 +263,33 @@ def test_data_roundtrip(pytestconfig, specimen, data_id, structure_id, tmp_path)
         pytest.param(
             "ISTAT/47_850-structure.xml", True, marks=[pytest.mark.skip(reason="Slow")]
         ),
-        pytest.param("IMF/ECOFIN_DSD-structure.xml", True, marks=_xf_ref),
+        ("IMF/ECOFIN_DSD-structure.xml", True),
         ("INSEE/CNA-2010-CONSO-SI-A17-structure.xml", False),
         ("INSEE/IPI-2010-A21-structure.xml", False),
-        pytest.param("INSEE/dataflow.xml", False, marks=_xf_not_equal),
+        ("INSEE/dataflow.xml", False),
         ("SGR/common-structure.xml", True),
         ("UNSD/codelist_partial.xml", True),
+        ("TEST/gh-149.xml", False),
     ],
 )
-def test_structure_roundtrip(pytestconfig, specimen, specimen_id, strict, tmp_path):
+def test_structure_roundtrip(specimen, specimen_id, strict, tmp_path):
     """Test that SDMX-ML StructureMessages can be 'round-tripped'."""
 
     # Read a specimen file
     with specimen(specimen_id) as f:
         msg0 = sdmx.read_sdmx(f)
 
-    # Write to file
-    path = tmp_path / "output.xml"
-    path.write_bytes(sdmx.to_xml(msg0, pretty_print=True))
+    # Write to a bytes buffer
+    data = io.BytesIO(sdmx.to_xml(msg0, pretty_print=True))
 
     # Read again
-    msg1 = sdmx.read_sdmx(path)
+    msg1 = sdmx.read_sdmx(data)
 
     # Contents are identical
-    assert msg0.compare(msg1, strict), (
-        path.read_text() if pytestconfig.getoption("verbose") else path
-    )
+    try:
+        assert msg0.compare(msg1, strict)
+    except AssertionError:  # pragma: no cover
+        path = tmp_path.joinpath("output.xml")
+        path.write_bytes(data.getbuffer())
+        log.error(f"compare() = False; see {path}")
+        raise
