@@ -1,11 +1,11 @@
 from itertools import chain
-from typing import Any, Dict, Hashable, Set, Union, cast
+from typing import Any, Dict, Hashable, Set, Union
 
 import numpy as np
 import pandas as pd
-from pandas.core.indexes.datetimes import prefix_mapping  # type: ignore [attr-defined]
 
 from sdmx import message
+from sdmx.dictlike import DictLike
 from sdmx.model import v21 as model
 from sdmx.model.v21 import (
     DEFAULT_LOCALE,
@@ -19,13 +19,14 @@ from sdmx.model.v21 import (
     SeriesKey,
     TimeDimension,
 )
-from sdmx.util import DictLike
 from sdmx.writer.base import BaseWriter
 
 #: Default return type for :func:`write_dataset` and similar methods. Either 'compat' or
 #: 'rows'. See the ref:`HOWTO <howto-rtype>`.
 DEFAULT_RTYPE = "rows"
 
+_HAS_PANDAS_2 = pd.__version__.split(".")[0] >= "2"
+
 
 writer = BaseWriter("pandas")
 
@@ -73,7 +74,7 @@ def _dict(obj: dict, *args, **kwargs):
             return DictLike(result)
     elif result_type == {str}:
         return pd.Series(result)
-    elif result_type == {DictLike}:
+    elif result_type < {dict, DictLike}:
         return result
     elif result_type == set():
         # No results
@@ -110,6 +111,7 @@ def write_datamessage(obj: message.DataMessage, *args, rtype=None, **kwargs):
         if `obj` has more than one data set.
     """
     # Pass the message's DSD to assist datetime handling
+    assert obj.dataflow
     kwargs.setdefault("dsd", obj.dataflow.structure)
 
     # Pass the return type and associated information
@@ -184,25 +186,28 @@ def _c(obj: model.Component):
 @writer
 def _cc(obj: model.ContentConstraint, **kwargs):
     """Convert :class:`.ContentConstraint`."""
-    if len(obj.data_content_region) != 1:
-        raise NotImplementedError
-
-    return writer.recurse(obj.data_content_region[0], **kwargs)
+    return {
+        i: writer.recurse(cr, **kwargs) for i, cr in enumerate(obj.data_content_region)
+    }
 
 
 @writer
 def _cr(obj: model.CubeRegion, **kwargs):
     """Convert :class:`.CubeRegion`."""
     result: DictLike[str, pd.Series] = DictLike()
-    for dim, memberselection in obj.member.items():
+    for dim, ms in obj.member.items():
         result[dim.id] = pd.Series(
-            # cast(): as of PR#30, only MemberValue is supported here
-            [cast(model.MemberValue, mv).value for mv in memberselection.values],
-            name=dim.id,
+            [writer.recurse(sv, **kwargs) for sv in ms.values], name=dim.id
         )
     return result
 
 
+@writer
+def _rp(obj: model.RangePeriod, **kwargs):
+    """Convert :class:`.RangePeriod`."""
+    return f"{obj.start.period}–{obj.end.period}"
+
+
 @writer
 def write_dataset(
     obj: model.DataSet,
@@ -218,12 +223,12 @@ def write_dataset(
 
     Parameters
     ----------
-    obj : :class:`~.DataSet` or iterable of :class:`~.Observation`
+    obj : :class:`~.DataSet` or iterable of :class:`Observation <.BaseObservation>`
     attributes : str
         Types of attributes to return with the data. A string containing zero or more
         of:
 
-        - ``'o'``: attributes attached to each :class:`~.Observation` .
+        - ``'o'``: attributes attached to each :class:`Observation <.BaseObservation>` .
         - ``'s'``: attributes attached to any (0 or 1) :class:`~.SeriesKey` associated
           with each Observation.
         - ``'g'``: attributes attached to any (0 or more) :class:`~.GroupKey` associated
@@ -311,7 +316,8 @@ def write_dataset(
 
     # NB mypy errors here on CI, but not locally
     result: Union[pd.Series, pd.DataFrame] = pd.DataFrame.from_dict(
-        data, orient="index"  # type: ignore [arg-type]
+        data,
+        orient="index",  # type: ignore [arg-type]
     )
 
     if len(result):
@@ -371,7 +377,7 @@ def _dataset_compat(df, datetime, kwargs):
     return df, datetime, kwargs
 
 
-def _maybe_convert_datetime(df, arg, obj, dsd=None):
+def _maybe_convert_datetime(df, arg, obj, dsd=None):  # noqa: C901
     """Helper for :meth:`.write_dataset` to handle datetime indices.
 
     Parameters
@@ -383,15 +389,14 @@ def _maybe_convert_datetime(df, arg, obj, dsd=None):
         From the `obj` argument to :meth:`write_dataset`.
     dsd: ~.DataStructureDefinition, optional
     """
-    # TODO simplify this method. It has McCabe complexity of 27; the maximum in the rest
-    #      of the code is 17.
-    if not arg:
-        # False, None, empty dict: no datetime conversion
-        return df
+    # TODO Simplify this method to reduce its McCabe complexity from 23 to <= 13
 
     # Check argument values
     param = dict(dim=None, axis=0, freq=False)
-    if isinstance(arg, str):
+
+    if not arg:
+        return df  # False, None, empty dict → no datetime conversion
+    elif isinstance(arg, str):
         param["dim"] = arg
     elif isinstance(arg, DimensionComponent):
         param["dim"] = arg.id
@@ -405,55 +410,48 @@ def _maybe_convert_datetime(df, arg, obj, dsd=None):
     else:
         raise ValueError(arg)
 
-    def _get_dims():
-        """Return an appropriate list of dimensions."""
-        if len(obj.structured_by.dimensions.components):
-            return obj.structured_by.dimensions.components
+    def _get(kind: str):
+        """Return an appropriate list of dimensions or attributes."""
+        if len(getattr(obj.structured_by, kind).components):
+            return getattr(obj.structured_by, kind).components
         elif dsd:
-            return dsd.dimensions.components
-        else:
-            return []
-
-    def _get_attrs():
-        """Return an appropriate list of attributes."""
-        if len(obj.structured_by.attributes.components):
-            return obj.structured_by.attributes.components
-        elif dsd:
-            return dsd.attributes.components
+            return getattr(dsd, kind).components
         else:
             return []
 
+    # Determine time dimension
     if not param["dim"]:
-        # Determine time dimension
-        dims = _get_dims()
-        for dim in dims:
-            if isinstance(dim, TimeDimension):
-                param["dim"] = dim
-                break
-        if not param["dim"]:
-            raise ValueError(f"no TimeDimension in {dims}")
+        for dim in filter(lambda d: isinstance(d, TimeDimension), _get("dimensions")):
+            param["dim"] = dim
+            break
+    if not param["dim"]:
+        raise ValueError(f"no TimeDimension in {_get('dimensions')}")
 
     # Unstack all but the time dimension and convert
     other_dims = list(filter(lambda d: d != param["dim"], df.index.names))
     df = df.unstack(other_dims)
-    df.index = pd.to_datetime(df.index, format="mixed")
+    # Only provide format in pandas >= 2.0.0
+    kw = dict(format="mixed") if _HAS_PANDAS_2 else {}
+    df.index = pd.to_datetime(df.index, **kw)
 
-    if param["freq"]:
-        # Determine frequency string, Dimension, or Attribute
-        freq = param["freq"]
-        if isinstance(freq, str) and freq not in prefix_mapping:
-            # ID of a Dimension or Attribute
-            for component in chain(_get_dims(), _get_attrs()):
+    # Convert to a PeriodIndex with a particular frequency
+    if freq := param["freq"]:
+        try:
+            # A frequency string recognized by pandas.PeriodDtype
+            if isinstance(freq, str):
+                freq = pd.PeriodDtype(freq=freq).freq
+        except ValueError:
+            # ID of a Dimension; Attribute; or column of `df`
+            result = None
+            for component in chain(
+                _get("dimensions"), _get("attributes"), map(Dimension, df.columns.names)
+            ):
                 if component.id == freq:
-                    freq = component
+                    freq = result = component
                     break
 
-            # No named dimension in the DSD; but perhaps on the df
-            if isinstance(freq, str):
-                if freq in df.columns.names:
-                    freq = Dimension(id=freq)
-                else:
-                    raise ValueError(freq)
+            if not result:
+                raise ValueError(freq)
 
         if isinstance(freq, Dimension):
             # Retrieve Dimension values from pd.MultiIndex level
@@ -462,9 +460,8 @@ def _maybe_convert_datetime(df, arg, obj, dsd=None):
             values = set(df.columns.levels[i])
 
             if len(values) > 1:
-                values = sorted(values)
                 raise ValueError(
-                    "cannot convert to PeriodIndex with " f"non-unique freq={values}"
+                    f"cannot convert to PeriodIndex with non-unique freq={sorted(values)}"
                 )
 
             # Store the unique value
