@@ -1,23 +1,27 @@
-"""SDMXML v2.1 writer."""
+"""SDMX-ML v2.1 writer."""
 # Contents of this file are organized in the order:
 #
 # - Utility methods and global variables.
 # - writer functions for sdmx.message classes, in the same order as message.py
 # - writer functions for sdmx.model classes, in the same order as model.py
 
-from typing import Iterable, List, cast
+import logging
+from typing import Iterable, List, Literal, cast
 
 from lxml import etree
 from lxml.builder import ElementMaker
 
 import sdmx.urn
 from sdmx import message
-from sdmx.format.xml import NS, qname, tag_for_class
+from sdmx.format.xml.v21 import NS, qname, tag_for_class
+from sdmx.model import common
 from sdmx.model import v21 as model
 from sdmx.writer.base import BaseWriter
 
 _element_maker = ElementMaker(nsmap={k: v for k, v in NS.items() if v is not None})
 
+log = logging.getLogger(__name__)
+
 writer = BaseWriter("XML")
 
 
@@ -41,10 +45,15 @@ def to_xml(obj, **kwargs):
     NotImplementedError
         If writing specific objects to SDMX-ML has not been implemented in :mod:`sdmx`.
     """
+    kwargs.setdefault("encoding", "utf-8")
+    kwargs.setdefault("xml_declaration", True)
     return etree.tostring(writer.recurse(obj), **kwargs)
 
 
-def reference(obj, parent=None, tag=None, style=None):
+RefStyle = Literal["Ref", "URN"]
+
+
+def reference(obj, parent=None, tag=None, *, style: RefStyle):
     """Write a reference to `obj`.
 
     .. todo:: Currently other functions in :mod:`.writer.xml` all pass the `style`
@@ -55,40 +64,51 @@ def reference(obj, parent=None, tag=None, style=None):
 
     elem = Element(tag)
 
-    if isinstance(obj, model.MaintainableArtefact):
-        ma = obj
-    else:
-        try:
-            # Get the ItemScheme for an Item
-            parent = parent or obj.get_scheme()
-        except AttributeError:  # pragma: no cover
-            # No `parent` and `obj` is not an Item with a .get_scheme() method
-            # NB this does not occur in the test suite
-            pass
-
-        if not parent:
-            raise NotImplementedError(
-                f"Cannot write reference to {repr(obj)} without parent"
-            )
-
-        ma = parent
-
+    # assert style
     if style == "URN":
         ref = Element(":URN", obj.urn)
     elif style == "Ref":
-        args = {
-            "id": obj.id,
-            "maintainableParentID": ma.id if parent else None,
-            "maintainableParentVersion": ma.version if parent else None,
-            "agencyID": getattr(ma.maintainer, "id", None),
-            "version": ma.version,
-            "package": model.PACKAGE[obj.__class__],
-            "class": etree.QName(tag_for_class(obj.__class__)).localname,
-        }
+        # Element attributes
+        attrib = dict(id=obj.id)
 
-        ref = Element(":Ref", **args)
-    else:  # pragma: no cover
-        raise ValueError(style)
+        # Identify a maintainable artifact; either `obj` or its `parent`
+        if isinstance(obj, model.MaintainableArtefact):
+            ma = obj
+            attrib.update(version=obj.version)
+        else:
+            try:
+                # Get the ItemScheme for an Item
+                parent = parent or obj.get_scheme()
+            except AttributeError:  # pragma: no cover
+                # No `parent` and `obj` is not an Item with a .get_scheme() method
+                # NB this does not occur in the test suite
+                pass
+
+            if not parent:
+                raise NotImplementedError(
+                    f"Cannot write reference to {obj!r} without parent"
+                )
+
+            ma = parent
+            attrib.update(
+                maintainableParentVersion=ma.version,
+                maintainableParentID=ma.id,
+            )
+
+        attrib.update(
+            agencyID=getattr(ma.maintainer, "id", None),
+            package=model.PACKAGE[type(ma).__name__],
+        )
+
+        # "class" attribute: either the type of `obj`, or the item type of an ItemScheme
+        for candidate in (type(obj), getattr(type(ma), "_Item", None)):
+            try:
+                attrib["class"] = etree.QName(tag_for_class(candidate)).localname
+                break
+            except ValueError:
+                pass
+
+        ref = Element(":Ref", **attrib)
 
     elem.append(ref)
     return elem
@@ -121,7 +141,7 @@ def _dm(obj: message.DataMessage):
             attrib["structureID"] = ds.structured_by.id
 
             # Reference by URN if possible, otherwise with a <Ref> tag
-            style = "URN" if ds.structured_by.urn else "Ref"
+            style: RefStyle = "URN" if ds.structured_by.urn else "Ref"
             dsd_ref = reference(ds.structured_by, tag="com:Structure", style=style)
 
             if isinstance(obj.observation_dimension, model.DimensionComponent):
@@ -167,11 +187,14 @@ def _sm(obj: message.StructureMessage):
         ("constraint", "Constraints"),
         ("provisionagreement", "ProvisionAgreements"),
     ]:
-        if not len(getattr(obj, attr)):
+        coll = getattr(obj, attr)
+        if not len(coll):
             continue
         container = Element(f"str:{tag}")
-        container.extend(writer.recurse(s) for s in getattr(obj, attr).values())
-        structures.append(container)
+        for s in filter(lambda s: not s.is_external_reference, coll.values()):
+            container.append(writer.recurse(s))
+        if len(container):
+            structures.append(container)
 
     if obj.footer:
         elem.append(writer.recurse(obj.footer))
@@ -250,29 +273,31 @@ def _a(obj: model.Annotation):
     elem = Element("com:Annotation")
     if obj.id:
         elem.attrib["id"] = obj.id
+    if obj.title:
+        elem.append(Element("com:AnnotationTitle", obj.title))
     if obj.type:
         elem.append(Element("com:AnnotationType", obj.type))
     elem.extend(i11lstring(obj.text, "com:AnnotationText"))
     return elem
 
 
-def annotable(obj, **kwargs):
-    cls = kwargs.pop("_tag", tag_for_class(obj.__class__))
+def annotable(obj, *args, **kwargs) -> etree._Element:
+    # Determine tag
+    tag = kwargs.pop("_tag", tag_for_class(obj.__class__))
+
+    # Write Annotations
+    e_anno = Element("com:Annotations", *[writer.recurse(a) for a in obj.annotations])
+    if len(e_anno):
+        args = args + (e_anno,)
+
     try:
-        elem = Element(cls, **kwargs)
+        return Element(tag, *args, **kwargs)
     except AttributeError:  # pragma: no cover
-        print(repr(obj), cls, kwargs)
+        print(repr(obj), tag, kwargs)
         raise
 
-    if len(obj.annotations):
-        e_anno = Element("com:Annotations")
-        e_anno.extend(writer.recurse(a) for a in obj.annotations)
-        elem.append(e_anno)
 
-    return elem
-
-
-def identifiable(obj, **kwargs):
+def identifiable(obj, *args, **kwargs) -> etree._Element:
     """Write :class:`.IdentifiableArtefact`.
 
     Unless the keyword argument `_with_urn` is :data:`False`, a URN is generated for
@@ -288,22 +313,25 @@ def identifiable(obj, **kwargs):
             kwargs.setdefault("urn", urn)
     except (AttributeError, ValueError):
         pass
-    return annotable(obj, **kwargs)
+    return annotable(obj, *args, **kwargs)
 
 
-def nameable(obj, **kwargs):
-    elem = identifiable(obj, **kwargs)
-    elem.extend(i11lstring(obj.name, "com:Name"))
-    elem.extend(i11lstring(obj.description, "com:Description"))
-    return elem
+def nameable(obj, *args, **kwargs) -> etree._Element:
+    return identifiable(
+        obj,
+        *i11lstring(obj.name, "com:Name"),
+        *i11lstring(obj.description, "com:Description"),
+        *args,
+        **kwargs,
+    )
 
 
-def maintainable(obj, **kwargs):
+def maintainable(obj, *args, **kwargs) -> etree._Element:
     kwargs.setdefault("version", obj.version)
     kwargs.setdefault("isExternalReference", str(obj.is_external_reference).lower())
     kwargs.setdefault("isFinal", str(obj.is_final).lower())
     kwargs.setdefault("agencyID", getattr(obj.maintainer, "id", None))
-    return nameable(obj, **kwargs)
+    return nameable(obj, *args, **kwargs)
 
 
 # §3.5: Item Scheme
@@ -319,7 +347,7 @@ def _item(obj: model.Item, **kwargs):
         e_parent.append(Element(":Ref", id=obj.parent.id, style="Ref"))
         elem.append(e_parent)
 
-    if isinstance(obj, model.Organisation):
+    if isinstance(obj, common.Organisation):
         elem.extend(writer.recurse(c) for c in obj.contact)
 
     return elem
@@ -345,9 +373,9 @@ def _facet(obj: model.Facet):
 
 
 @writer
-def _rep(obj: model.Representation, tag, style="URN"):
+def _rep(obj: common.Representation, tag, style="URN"):
     elem = Element(f"str:{tag}")
-    if obj.enumerated:
+    if obj.enumerated is not None:
         elem.append(reference(obj.enumerated, tag="str:Enumeration", style=style))
     if obj.non_enumerated:
         elem.extend(writer.recurse(facet) for facet in obj.non_enumerated)
@@ -378,8 +406,8 @@ def _contact(obj: model.Contact):
         + i11lstring(obj.org_unit, "str:Department")
         + i11lstring(obj.responsibility, "str:Role")
         + ([Element("str:Telephone", obj.telephone)] if obj.telephone else [])
-        + [Element("str:URI", text=value) for value in obj.uri]
-        + [Element("str:Email", text=value) for value in obj.email]
+        + [Element("str:URI", value) for value in obj.uri]
+        + [Element("str:Email", value) for value in obj.email]
     )
     return elem
 
@@ -388,34 +416,41 @@ def _contact(obj: model.Contact):
 
 
 @writer
-def _component(obj: model.Component):
-    elem = identifiable(obj)
-    if obj.concept_identity:
-        elem.append(
+def _component(obj: model.Component, dsd):
+    child = []
+    attrib = dict()
+
+    try:
+        child.append(
             reference(obj.concept_identity, tag="str:ConceptIdentity", style="Ref")
         )
-    if obj.local_representation:
-        elem.append(
+    except AttributeError:  # pragma: no cover
+        pass  # concept_identity is None
+
+    try:
+        child.append(
             writer.recurse(obj.local_representation, "LocalRepresentation", style="Ref")
         )
-    # DataAttribute only
-    try:
-        elem.append(writer.recurse(cast(model.DataAttribute, obj).related_to))
-    except AttributeError:
-        pass
-    except NotImplementedError:  # pragma: no cover
-        if getattr(obj, "related_to", None) is None:
-            pass  # .related_to not set
-        else:
-            raise  # Some other NotImplementedError
+    except NotImplementedError:
+        pass  # None
 
-    return elem
+    if isinstance(obj, model.DataAttribute) and obj.usage_status:
+        child.append(writer.recurse(obj.related_to, dsd))
+
+        # assignmentStatus attribute
+        if obj.usage_status:
+            attrib["assignmentStatus"] = obj.usage_status.name.title()
+    elif isinstance(obj, model.Dimension):
+        # position attribute
+        attrib["position"] = str(obj.order)
+
+    return identifiable(obj, *child, **attrib)
 
 
 @writer
-def _cl(obj: model.ComponentList):
+def _cl(obj: model.ComponentList, *args):
     elem = identifiable(obj)
-    elem.extend(writer.recurse(c) for c in obj.components)
+    elem.extend(writer.recurse(c, *args) for c in obj.components)
     return elem
 
 
@@ -455,7 +490,12 @@ def _dks(obj: model.DataKeySet):
 
 @writer
 def _ms(obj: model.MemberSelection):
-    elem = Element("com:KeyValue", id=obj.values_for.id)
+    tag = {
+        model.Dimension: "KeyValue",
+        model.DataAttribute: "Attribute",
+    }[type(obj.values_for)]
+
+    elem = Element(f"com:{tag}", id=obj.values_for.id)
     elem.extend(
         # cast(): as of PR#30, only MemberValue is supported here
         Element("com:Value", cast(model.MemberValue, mv).value)
@@ -473,6 +513,7 @@ def _cr(obj: model.CubeRegion):
 
 @writer
 def _cc(obj: model.ContentConstraint):
+    assert obj.role is not None
     elem = maintainable(
         obj, type=obj.role.role.name.replace("allowable", "allowed").title()
     )
@@ -496,22 +537,26 @@ def _cc(obj: model.ContentConstraint):
 
 
 @writer
-def _nsr(obj: model._NoSpecifiedRelationship):
+def _nsr(obj: model.NoSpecifiedRelationship, *args):
     elem = Element("str:AttributeRelationship")
     elem.append(Element("str:None"))
     return elem
 
 
 @writer
-def _pmr(obj: model._PrimaryMeasureRelationship):
+def _pmr(obj: model.PrimaryMeasureRelationship, dsd: model.DataStructureDefinition):
     elem = Element("str:AttributeRelationship")
     elem.append(Element("str:PrimaryMeasure"))
-    elem[-1].append(Element(":Ref", id="(not implemented)"))
+    try:
+        ref_id = dsd.measures[0].id
+    except IndexError:
+        ref_id = "(not implemented)"  # MeasureDescriptor is empty
+    elem[-1].append(Element(":Ref", id=ref_id))
     return elem
 
 
 @writer
-def _dr(obj: model.DimensionRelationship):
+def _dr(obj: common.DimensionRelationship, *args):
     elem = Element("str:AttributeRelationship")
     for dim in obj.dimensions:
         elem.append(Element("str:Dimension"))
@@ -520,7 +565,7 @@ def _dr(obj: model.DimensionRelationship):
 
 
 @writer
-def _gr(obj: model.GroupRelationship):
+def _gr(obj: common.GroupRelationship, *args):
     elem = Element("str:AttributeRelationship")
     elem.append(Element("str:Group"))
     elem[-1].append(Element(":Ref", id=getattr(obj.group_key, "id", None)))
@@ -543,11 +588,11 @@ def _dsd(obj: model.DataStructureDefinition):
     elem.append(Element("str:DataStructureComponents"))
 
     # Write in a specific order
-    elem[-1].append(writer.recurse(obj.dimensions))
+    elem[-1].append(writer.recurse(obj.dimensions, None))
     for group in obj.group_dimensions.values():
         elem[-1].append(writer.recurse(group))
-    elem[-1].append(writer.recurse(obj.attributes))
-    elem[-1].append(writer.recurse(obj.measures))
+    elem[-1].append(writer.recurse(obj.attributes, obj))
+    elem[-1].append(writer.recurse(obj.measures, None))
 
     return elem
 
